require "import"
import "android.net.Uri"
import "android.content.Intent"
import "android.provider.ContactsContract"
import "android.widget.*"
import "android.view.*"
import "java.util.*"
import "android.content.pm.PackageManager"
import "android.os.Build"
import "android.Manifest"
import "android.text.TextWatcher"
import "android.widget.EditText"

-- Runtime permission request codes
local CONTACTS_PERMISSION_CODE = 100
local CALL_PERMISSION_CODE = 101
local permissions = {
    ["READ_CONTACTS"] = Manifest.permission.READ_CONTACTS,
    ["CALL_PHONE"] = Manifest.permission.CALL_PHONE
}

local activity = activity
local listView
local contactsList = ArrayList()
local numbersList = {}
local searchEditText

function onCreate(savedInstanceState)
    -- Set up layout
    local layout = LinearLayout(activity)
    layout.setOrientation(LinearLayout.VERTICAL)
    
    searchEditText = EditText(activity)
    searchEditText.setHint("Search contacts")
    layout.addView(searchEditText)
    
    listView = ListView(activity)
    layout.addView(listView)
    
    activity.setContentView(layout)
    activity.setTitle("Contacts List")
    
    -- Set up search listener
    searchEditText.addTextChangedListener(TextWatcher{
        onTextChanged = function(s, start, before, count)
            filterContacts(s.toString())
        end
    })
    
    -- Check for contacts permission, then load contacts
    if not hasPermission(permissions["READ_CONTACTS"]) then
        requestPermission(permissions["READ_CONTACTS"], CONTACTS_PERMISSION_CODE)
    else
        loadContacts()
    end
    
    -- Hide the soft keyboard initially
    activity.getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_HIDDEN)
end

function hasPermission(permission)
    if Build.VERSION.SDK_INT >= 23 then
        return activity.checkSelfPermission(permission) == PackageManager.PERMISSION_GRANTED
    end
    return true
end

function requestPermission(permission, requestCode)
    if Build.VERSION.SDK_INT >= 23 then
        activity.requestPermissions({permission}, requestCode)
    end
end

function onRequestPermissionsResult(requestCode, permissions, grantResults)
    if requestCode == CONTACTS_PERMISSION_CODE then
        if grantResults[1] == PackageManager.PERMISSION_GRANTED then
            loadContacts()
        else
            Toast.makeText(activity, "Permission Denied", Toast.LENGTH_SHORT).show()
        end
    elseif requestCode == CALL_PERMISSION_CODE then
        if grantResults[1] == PackageManager.PERMISSION_GRANTED then
            makePhoneCall(numbersList[listView.getSelectedItemPosition()])
        else
            Toast.makeText(activity, "Calling Permission Denied", Toast.LENGTH_SHORT).show()
        end
    end
end

function loadContacts()
    contactsList.clear()
    numbersList = {}
    local cursor = activity.getContentResolver().query(
        ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
        nil, nil, nil, ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME.." ASC"
    )
    while cursor.moveToNext() do
        local name = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME))
        local phoneNumber = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER))
        contactsList.add(name .. " (" .. phoneNumber .. ")")
        table.insert(numbersList, phoneNumber)
    end
    cursor.close()

    local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, contactsList)
    listView.setAdapter(adapter)

    -- Set on item click listener to make phone call
    listView.setOnItemClickListener(AdapterView.OnItemClickListener{
        onItemClick = function(adapterView, view, position, id)
            if not hasPermission(permissions["CALL_PHONE"]) then
                requestPermission(permissions["CALL_PHONE"], CALL_PERMISSION_CODE)
            else
                makePhoneCall(numbersList[position + 1]) -- Offset index because Lua is 1-indexed
            end
        end
    })
end

function makePhoneCall(phoneNumber)
    local callIntent = Intent(Intent.ACTION_CALL)
    callIntent.setData(Uri.parse("tel:" .. phoneNumber))
    activity.startActivity(callIntent)
end

function filterContacts(query)
    local filteredList = ArrayList()
    for i = 0, contactsList.size() - 1 do
        local contact = contactsList.get(i)
        if contact:lower():find(query:lower(), 1, true) then
            filteredList.add(contact)
        end
    end
    local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, filteredList)
    listView.setAdapter(adapter)
end